package com.xubx.springboot_01demo.Task;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xubx.springboot_01demo.configuration.constantConfiguration;
import com.xubx.springboot_01demo.mapper.BlogLikeMapper;
import com.xubx.springboot_01demo.mapper.BlogsMapper;
import com.xubx.springboot_01demo.mapper.UserFavoriteBlogMapper;
import com.xubx.springboot_01demo.pojo.BlogLike;
import com.xubx.springboot_01demo.pojo.Blogs;
import com.xubx.springboot_01demo.pojo.UserFavoriteBlog;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class DataSync {
    @Resource
    RedisTemplate redisTemplate;
    @Resource
    BlogsMapper blogsMapper;
    @Resource
    BlogLikeMapper blogLikeMapper;
    @Resource
    UserFavoriteBlogMapper userFavoriteBlogMapper;
    @Autowired
    ObjectMapper objectMapper;

    /**
     * 博客浏览数的数据同步 ，每小时执行一次
     */
//    @Scheduled(cron = "0 0 * * * ?")
    @Scheduled(cron = "0 * * * * ?")
    public void viewCountSync() {
        log.info("博客浏览数的数据同步定时任务执行");

        String lockKey = constantConfiguration.BLOG_VIEW_COUNT_SYNC_LOCK;
        Boolean isLock = Boolean.TRUE.equals(redisTemplate.opsForValue().setIfAbsent(lockKey, "1", 60, TimeUnit.SECONDS));
        if (Boolean.TRUE.equals(isLock)) {
            try {
                Set<String> viewCountKeys = redisTemplate.keys(constantConfiguration.BLOG_VIEW_COUNT + "*");
                if (viewCountKeys != null) {
                    for (String key : viewCountKeys) {
                        String blogIdStr = key.split(":")[2];
                        String expirekey = constantConfiguration.BLOG_VIEW_COUNT + blogIdStr + ":expired";

                        Boolean isExpired = Boolean.TRUE.equals(redisTemplate.hasKey(expirekey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(expirekey));
                        // 如果标签存在 则进行数据的更新，若不存在则表示数据已经同步 或者数据还没被更新
                        if (Boolean.TRUE.equals(isExpired)) {
                            String viewCount = (String) redisTemplate.opsForValue().get(key);
                            if (viewCount != null) {
                                //更新数据
                                LambdaUpdateWrapper<Blogs> updateWrapper = new LambdaUpdateWrapper<>();
                                updateWrapper.eq(Blogs::getId, blogIdStr)
                                        .set(Blogs::getViewCount, viewCount);
                                blogsMapper.update(null, updateWrapper);
                                // 删除逻辑过期标志
                                redisTemplate.delete(expirekey);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，博客浏览数的数据同步任务无法执行");
        }
    }

    /**
     * 博客点赞数的数据同步定时任务执行
     */
    @Scheduled(cron = "0 * * * * ?")
    public void likeCountSync() {
        log.info("博客点赞数的数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_LIKE_COUNT_SYNC_LOCK;
        Boolean isLock = Boolean.TRUE.equals(redisTemplate.opsForValue().setIfAbsent(lockKey, "1", 60, TimeUnit.SECONDS));

        if (Boolean.TRUE.equals(isLock)) {
            try {
                Set<String> likeCountKeys = redisTemplate.keys(constantConfiguration.BLOG_LIKE_COUNT + "*");
                if (likeCountKeys != null) {
                    for (String key : likeCountKeys) {
                        String blogIdStr = key.split(":")[2];
                        String likeCountExpireKey = constantConfiguration.BLOG_LIKE_COUNT + blogIdStr + ":expired";

                        Boolean countExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(likeCountExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(likeCountExpireKey));

                        if (Boolean.TRUE.equals(countExpireFlag)) {
                            String likeCount = (String) redisTemplate.opsForValue().get(key);
                            if (likeCount != null) {
                                // 更新点赞次数到数据库
                                LambdaUpdateWrapper<Blogs> updateWrapper = new LambdaUpdateWrapper<>();
                                updateWrapper.eq(Blogs::getId, blogIdStr).set(Blogs::getLikeCount, Integer.parseInt(likeCount));
                                blogsMapper.update(null, updateWrapper);

                                // 删除逻辑过期标志
                                redisTemplate.delete(likeCountExpireKey);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，博客点赞数的数据同步任务无法执行");
        }
    }

    /**
     * 博客点赞用户Hash集合的增量数据同步定时任务执行
     */
    @Scheduled(cron = "0 * * * * ?") //TODO 每小时执行一次
    public void likeUserSync() {
        log.info("博客点赞用户Hash集合的增量数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_LIKE_USERHASH_SYNC_LOCK;
        Boolean isLock = Boolean.TRUE.equals(redisTemplate.opsForValue().setIfAbsent(lockKey, "1", 60, TimeUnit.SECONDS));

        if (Boolean.TRUE.equals(isLock)) {
            try {
                Set<String> likeUserKeys = redisTemplate.keys(constantConfiguration.BLOG_LIKE_HASH + "*");
                if (likeUserKeys != null) {
                    for (String key : likeUserKeys) {
                        String blogIdStr = key.split(":")[2];
                        String likeUserExpireKey = constantConfiguration.BLOG_LIKE_HASH + blogIdStr + ":expired";

                        Boolean userHashExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(likeUserExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(likeUserExpireKey));

                        if (Boolean.TRUE.equals(userHashExpireFlag)) {
                            // 获取博客点赞用户集合中的所有用户ID
                            Set<String> userIds = redisTemplate.opsForHash().keys(constantConfiguration.BLOG_LIKE_HASH + blogIdStr);
                            if (userIds != null) {
                                for (String userId : userIds) {
                                    String likeStatus = (String) redisTemplate.opsForHash().get(constantConfiguration.BLOG_LIKE_HASH + blogIdStr, userId);
                                    if ("true".equals(likeStatus)) {
                                        // 插入数据
                                        LambdaQueryWrapper<BlogLike> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(BlogLike::getBlogId, blogIdStr).eq(BlogLike::getUserId, userId);
                                        BlogLike blogLike = blogLikeMapper.selectOne(queryWrapper);

                                        if (blogLike == null) {
                                            BlogLike newBlogLike = new BlogLike();
                                            newBlogLike.setBlogId(Integer.valueOf(blogIdStr));
                                            newBlogLike.setUserId(Integer.valueOf(userId));
                                            newBlogLike.setCreatedTime(Timestamp.valueOf(LocalDateTime.now()));
                                            blogLikeMapper.insert(newBlogLike);
                                        }
                                    } else if ("false".equals(likeStatus)) {
                                        // 用户取消点赞 则删除记录
                                        LambdaQueryWrapper<BlogLike> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(BlogLike::getBlogId, blogIdStr).eq(BlogLike::getUserId, userId);
                                        blogLikeMapper.delete(queryWrapper);
                                    }
                                }
                                // 删除逻辑过期标志和哈希数据
                                redisTemplate.delete(likeUserExpireKey);
                                redisTemplate.delete(constantConfiguration.BLOG_LIKE_HASH + blogIdStr);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，博客点赞用户Hash集合的增量数据同步任务无法执行");
        }
    }

    /**
     * 博客收藏数的数据同步定时任务
     */
    @Scheduled(cron = "0 * * * * ?") //TODO 每小时执行一次
    public void collectCountSync() {
        log.info("博客收藏数的数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_COLLECT_COUNT_SYNC_LOCK;
        Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(lockKey, "LOCKED", 30, TimeUnit.MINUTES);

        if (Boolean.TRUE.equals(lockAcquired)) {
            try {
                Set<String> collectCountKeys = redisTemplate.keys(constantConfiguration.BLOG_COLLECT_COUNT + "*");
                if (collectCountKeys != null) {
                    for (String key : collectCountKeys) {
                        String blogIdStr = key.split(":")[2];
                        String collectCountExpireKey = constantConfiguration.BLOG_COLLECT_COUNT + blogIdStr + ":expired";

                        Boolean countExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(collectCountExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(collectCountExpireKey));

                        if (Boolean.TRUE.equals(countExpireFlag)) {
                            String collectCount = (String) redisTemplate.opsForValue().get(key);
                            if (collectCount != null) {
                                // 更新收藏次数到数据库
                                LambdaUpdateWrapper<Blogs> updateWrapper = new LambdaUpdateWrapper<>();
                                updateWrapper.eq(Blogs::getId, blogIdStr).set(Blogs::getCollectCount, Integer.parseInt(collectCount));
                                blogsMapper.update(null, updateWrapper);

                                // 删除逻辑过期标志
                                redisTemplate.delete(collectCountExpireKey);
                            }
                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，跳过此次收藏数同步任务");
        }
    }

    /**
     * 博客收藏增量数据同步
     *
     * @throws JsonProcessingException
     */
    @Scheduled(cron = "0 * * * * ?") //TODO 每小时执行一次
    public void collectUserSync() {
        log.info("博客收藏用户Hash集合的数据同步定时任务执行");
        String lockKey = constantConfiguration.BLOG_COLLECT_USERHASH_SYNC_LOCK;
        Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(lockKey, "LOCKED", 30, TimeUnit.MINUTES);

        if (Boolean.TRUE.equals(lockAcquired)) {
            try {
                // TODO 应该是同时获取到了逻辑过期标志的Key，需要进行去除
                Set<String> collectUserKeys = redisTemplate.keys(constantConfiguration.BLOG_COLLECT_HASH + "*");
                if (collectUserKeys != null) {
                    for (String key : collectUserKeys) {
                        String favoriteIdStr = key.split(":")[2];
                        String collectUserExpireKey = constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr + ":expired";

                        Boolean userExpireFlag = Boolean.TRUE.equals(redisTemplate.hasKey(collectUserExpireKey))
                                && Boolean.parseBoolean((String) redisTemplate.opsForValue().get(collectUserExpireKey));
                        if (Boolean.TRUE.equals(userExpireFlag)) {
                            // 获取博客收藏用户集合中的所有博客
                            Set<String> blogSet = redisTemplate.opsForHash().keys(constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr);
                            if (blogSet != null) {
                                for (String blogIdStr : blogSet) {
                                    // 根据用户的收藏状态，更新数据
                                    String collectStatus = (String) redisTemplate.opsForHash().get(constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr, blogIdStr);
                                    if ("true".equals(collectStatus)) {
                                        // 如果数据库中没有该数据则插入数据
                                        LambdaQueryWrapper<UserFavoriteBlog> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(UserFavoriteBlog::getBlogId, blogIdStr).eq(UserFavoriteBlog::getFavoriteId, favoriteIdStr);
                                        UserFavoriteBlog userFavoriteBlog = userFavoriteBlogMapper.selectOne(queryWrapper);
                                        if (userFavoriteBlog == null) {
                                            userFavoriteBlog = new UserFavoriteBlog();
                                            userFavoriteBlog.setBlogId(Integer.valueOf(blogIdStr));
                                            userFavoriteBlog.setFavoriteId(Integer.valueOf(favoriteIdStr));
                                            userFavoriteBlogMapper.insert(userFavoriteBlog);
                                        }
                                    } else if ("false".equals(collectStatus)) {
                                        // 用户取消收藏 删除数据
                                        LambdaQueryWrapper<UserFavoriteBlog> queryWrapper = new LambdaQueryWrapper<>();
                                        queryWrapper.eq(UserFavoriteBlog::getBlogId, blogIdStr).eq(UserFavoriteBlog::getFavoriteId, favoriteIdStr);
                                        userFavoriteBlogMapper.delete(queryWrapper);
                                    }
                                }
                                // 删除逻辑过期标志和Hash集合
                                redisTemplate.delete(collectUserExpireKey);
                                redisTemplate.delete(constantConfiguration.BLOG_COLLECT_HASH + favoriteIdStr);
                            }

                        }
                    }
                }
            } finally {
                redisTemplate.delete(lockKey);
            }
        } else {
            log.warn("无法获取分布式锁，跳过此次收藏用户Hash集合的数据同步任务");
        }
    }
}
